import QtQuick 1.1
import com.victron.velib 1.0
import "utils.js" as Utils

MbPage {
	id: root
	title: qsTr("Generator start/stop")
	property string settingsBindPrefix: "com.victronenergy.settings"
	property string startStopBindPrefix: "com.victronenergy.generator.startstop0"
	property VBusItem activeCondition: VBusItem { bind: Utils.path(startStopBindPrefix, "/RunningByCondition") }
	property VBusItem generatorState: VBusItem { bind: Utils.path(startStopBindPrefix, "/State") }
	property VBusItem runningTime: VBusItem { bind: Utils.path(startStopBindPrefix, "/Runtime") }
	property VBusItem historicalData: VBusItem { bind: Utils.path(settingsBindPrefix, "/Settings/Generator0/AccumulatedDaily") }
	property VBusItem relayFunction: VBusItem { bind: Utils.path(settingsBindPrefix, "/Settings/Relay/Function") }

	function getConditionName(condition)
	{
		switch(condition) {
		case 'soc':
			return qsTr("Running by SOC condition")
		case 'acload':
			return qsTr("Running by AC Load condition")
		case 'batterycurrent':
			return qsTr("Running by battery current condition")
		case 'batteryvoltage':
			return qsTr("Running by battery voltage condition")
		case 'inverterhightemp':
			return qsTr("Running by inverter high temperature")
		case 'inverteroverload':
			return qsTr("Running by inverter overload")
		case 'testrun':
			return qsTr("Test run")
		case 'lossofcommunication':
			return qsTr("Running by loss of communication")
		case 'manual':
			return qsTr("Manually started")
		default:
			return qsTr("Stopped")
		}
	}

	model: relayFunction.value == undefined ? startStopModel : relayFunction.value  == 1 ? startStopModel : disabledModel

	VisualItemModel {
		id: disabledModel

		MbItemText {
			wrapMode: Text.WordWrap
			text: qsTr("Generator start/stop function is not enabled, go to relay settings and set " +
					   "function to \"Generator start/stop\"" )
		}
	}

	VisualItemModel {
		id: startStopModel

		MbItemValue {
			description: qsTr("Generator state")
			item.text: activeCondition.valid ? getConditionName(activeCondition.value) : '---'
		}

		MbItemValue {
			description: qsTr("Run time")
			item.text: runningTime.valid ? Utils.secondsToString(runningTime.value) : "0"
			show: generatorState.value === 1
		}

		MbItemValue {
			description: qsTr("Total run time")
			item {
				bind: Utils.path(settingsBindPrefix, "/Settings/Generator0/AccumulatedTotal")
				text: Utils.secondsToString(item.value)
			}
		}

		MbItemValue {
			description: qsTr("Accumulated running time since last test run")
			show: ColorControl.accessLevel >= Victron.AccessService && nextTestRun.show
			item {
				text: Utils.secondsToString(item.value)
				bind: Utils.path(startStopBindPrefix, "/TestRunIntervalRuntime")
			}
		}

		MbItemValue {
			id: nextTestRun
			description: qsTr("Time to next test run")
			show: item.valid && item.value > 0
			item {
				text: {
					var remainingTime = item.value - new Date().getTime() / 1000
					if (remainingTime > 0)
						return Utils.secondsToString(remainingTime).toString()
					return qsTr("Running now")
				}
				bind: Utils.path(startStopBindPrefix, "/NextTestRun")
			}
		}

		MbSwitch {
			name: qsTr("Auto start functionality")
			bind: Utils.path(settingsBindPrefix, "/Settings/Generator0/AutoStartEnabled")
		}

		MbSubMenu {
			description: qsTr("Manual start")
			subpage: Component {
				MbPage {
					id: manualStartPage
					title: qsTr("Manual start")
					model: VisualItemModel {
						MbSwitch {
							id: manualSwitch
							name: qsTr("Start generator")
							bind: Utils.path(startStopBindPrefix, "/ManualStart")
							writeAccessLevel: Victron.AccessUser
							onCheckedChanged: {
								if (manualStartPage.active) {
									if (!checked)
										toast.createToast(qsTr("Stopping, generator will continue running if other conditions are reached"), 3000)
									if (checked && stopTimer.value == 0)
										toast.createToast(qsTr("Starting, generator won't stop till user intervention"), 5000)
									if (checked && stopTimer.value > 0)
										toast.createToast(qsTr("Starting, generator will stop in %1 if no other conditions are reached").arg(Utils.secondsToString(stopTimer.value)), 5000)
								}
							}

							VBusItem {
								id: stopTimer
								bind: Utils.path(startStopBindPrefix, "/ManualStartTimer")
							}
						}

						MbEditBoxTime {
							description: qsTr("Run for (hh:mm)")
							readonly: manualSwitch.checked
							bind: Utils.path(startStopBindPrefix, "/ManualStartTimer")
							writeAccessLevel: Victron.AccessUser
						}
					}
				}
			}
		}

		MbSubMenu {
			description: qsTr("Daily run time")
			subpage: MbPage {
				// Invert the order
				property variant keys: historicalData.valid ?
										Object.keys(JSON.parse(historicalData.value)).reverse() : 0

				title: qsTr("Daily run time")
				focus: active
				model: keys
				delegate: MbItemValue {
					description: Qt.formatDate(new Date(parseInt(keys[index]) * 1000), "dd-MM-yyyy");
					item.text: Utils.secondsToString(JSON.parse(historicalData.value)[keys[index]])
				}
			}
		}

		MbSubMenu {
			id: conditions
			description: qsTr("Settings")
			subpage: Component {
				PageSettingsGenerator {
					settingsBindPrefix: root.settingsBindPrefix
				}
			}
		}
	}
}
