import QtQuick 1.1
import com.victron.velib 1.0
import "utils.js" as Utils

MbPage {
	property string cgwacsPath: "com.victronenergy.settings/Settings/CGwacs"
	property string batteryLifePath: cgwacsPath + "/BatteryLife"
	property int hub4PhaseCompensation: 1
	property int hub4PhaseSplit: 2
	property int hub4Disabled: 3
	property bool initialized: false

	Component.onCompleted: {
		initialized = true
	}

	VBusItem {
		id: maxChargePowerItem
		bind: Utils.path(cgwacsPath, "/MaxChargePower")
	}

	VBusItem {
		id: maxDischargePowerItem
		bind: Utils.path(cgwacsPath, "/MaxDischargePower")
	}

	VBusItem {
		id: socLimitItem
		bind: Utils.path(batteryLifePath, "/SocLimit")
	}

	VBusItem {
		id: minSocLimitItem
		bind: Utils.path(batteryLifePath, "/MinimumSocLimit")
	}

	VBusItem {
		id: stateItem
		bind: Utils.path(batteryLifePath, "/State")
	}

	VBusItem {
		id: hub4Mode
		bind: Utils.path(cgwacsPath, "/Hub4Mode")
	}

	VBusItem {
		id: vebusPath
		bind: "com.victronenergy.system/Hub"
	}

	model: vebusPath.value === 4 ? hub4Settings : noHub4

	VisualItemModel {
		id: noHub4

		MbItemText {
			text: qsTr("Hub4 assistant not found")
		}
	}

	VisualItemModel {
		id: hub4Settings

		MbSwitch {
			name: qsTr("Hub-4 control")
			checked: hub4Mode.value !== hub4Disabled
			enabled: userHasWriteAccess
			onCheckedChanged: {
				if (checked && hub4Mode.value === hub4Disabled)
					hub4Mode.setValue(hub4PhaseCompensation)
				else if (!checked && hub4Mode.value !== hub4Disabled)
					hub4Mode.setValue(hub4Disabled)
			}
		}

		MbSwitch {
			name: qsTr("Control without grid meter")
			bind: Utils.path(cgwacsPath, '/RunWithoutGridMeter')
			show: hub4Mode.value !== hub4Disabled
			enabled: userHasWriteAccess
		}

		MbSwitch {
			name: qsTr("Phase compensation")
			bind: hub4Mode.bind
			valueTrue: hub4PhaseCompensation
			valueFalse: hub4PhaseSplit
			show: hub4Mode.value !== hub4Disabled
			enabled: userHasWriteAccess
			onCheckedChanged: {
				if (initialized && !checked)
					toast.createToast(qsTr("Disabling phase compensation may cause additional energy loss, because extra power has to be sent over DC cables between Multis/Quattros to balance all phases. Disable phase compensation only if your electricy provider does not allow it."), 15000);
			}
		}

		MbSpinBox {
			id: minSocLimit
			description: qsTr("Minimum Discharge SoC (unless grid fails)")
			enabled: userHasWriteAccess
			show: hub4Mode.value !== hub4Disabled
			bind: Utils.path(batteryLifePath, "/MinimumSocLimit")
			numOfDecimals: 0
			unit: "%"
			min: 0
			max: 100
			stepSize: 5
		}

		MbSwitch {
			// Overload the edit function in MbSwitch, the edit function will only be called
			// whenever the switch is toggled by the user, unlike the onCheckedChanged signal which
			// is also emited when stateItem is changed on the D-Bus.
			function edit() {
				if (enabled) {
					if (stateItem.value === 0)
						stateItem.setValue(1)
					else if (stateItem.value > 0)
						stateItem.setValue(0)
				}
			}
			name: qsTr("Battery life")
			checked: stateItem.value > 1
			enabled: userHasWriteAccess
			show: hub4Mode.value !== hub4Disabled
		}

		MbItemValue {
			id: socLimit
			description: qsTr("Actual state of charge limit")
			show: hub4Mode.value !== hub4Disabled && stateItem.value > 1
			item.value: Math.max(minSocLimitItem.value, socLimitItem.value)
			item.unit: '%'
		}

		MbItemOptions {
			description: qsTr("Battery life state")
			value: stateItem.value
			readonly: true
			show: hub4Mode.value !== hub4Disabled && stateItem.value > 1
			possibleValues:[
				// Values below taken from MaintenanceState enum in dbus-cgwacs
				MbOption { description: qsTr("Self consumption"); value: 2 },
				MbOption { description: qsTr("Self consumption"); value: 3 },
				MbOption { description: qsTr("Self consumption"); value: 4 },
				MbOption { description: qsTr("Discharge disabled"); value: 5 },
				MbOption { description: qsTr("Force charge"); value: 6 }
			]
		}

		MbSwitch {
			name: qsTr("Limit charge power")
			checked: maxChargePowerItem.value >= 0
			enabled: userHasWriteAccess
			show: hub4Mode.value !== hub4Disabled
			onCheckedChanged: {
				if (checked && maxChargePowerItem.value < 0)
					maxChargePowerItem.setValue(1000)
				else if (!checked && maxChargePowerItem.value >= 0)
					maxChargePowerItem.setValue(-1)
			}
		}

		MbSpinBox {
			id: maxChargePower
			description: qsTr("Maximum charge power")
			enabled: userHasWriteAccess
			show: hub4Mode.value !== hub4Disabled && maxChargePowerItem.value >= 0
			bind: Utils.path(cgwacsPath, "/MaxChargePower")
			numOfDecimals: 0
			unit: "W"
			min: 0
			max: 50000
			stepSize: 50
		}

		MbSwitch {
			name: qsTr("Limit discharge power")
			checked: maxDischargePowerItem.value >= 0
			enabled: userHasWriteAccess
			show: hub4Mode.value !== hub4Disabled
			onCheckedChanged: {
				if (checked && maxDischargePowerItem.value < 0)
					maxDischargePowerItem.setValue(1000)
				else if (!checked && maxDischargePowerItem.value >= 0)
					maxDischargePowerItem.setValue(-1)
			}
		}

		MbSpinBox {
			id: maxDischargePower
			description: qsTr("Maximum discharge power")
			enabled: userHasWriteAccess
			show: hub4Mode.value !== hub4Disabled && maxDischargePowerItem.value >= 0
			bind: Utils.path(cgwacsPath, "/MaxDischargePower")
			numOfDecimals: 0
			unit: "W"
			min: 0
			max: 50000
			stepSize: 50
		}

		MbSwitch {
			name: qsTr("Zero feed-in (Fronius PV inverters only)")
			bind: Utils.path(cgwacsPath, "/PreventFeedback")
			show: hub4Mode.value !== hub4Disabled
			enabled: userHasWriteAccess
		}

		MbSubMenu {
			id: deviceItem
			description: qsTr("Hub-4 Grid Setpoint")
			show: hub4Mode.value !== hub4Disabled && ColorControl.accessLevel >= Victron.AccessSuperUser
			subpage: Component {
				PageHub4Debug {
					title: qsTr("Hub-4 Grid Setpoint")
				}
			}
		}
	}
}
