import QtQuick 1.1
import com.victron.velib 1.0

MbPage {
	id: root

	property variant service
	property string bindPrefix

	title: service.description
	summary: yield.item.text

	VBusItem {
		id: productName
		bind: service.path("/ProductName")
	}

	VBusItem {
		id: productId
		bind: service.path("/ProductId")
	}

	VBusItem {
		id: firmwareVersion
		bind: service.path("/FirmwareVersion")
	}

	function isModelSupported()
	{
		if (!productId.valid || !firmwareVersion.valid)
			return true

		/* MPPT 70/15 (product id 0x300) has limited VE.Text support */
		if (productId.value === 0x300)
			return false

		/* Reserved space for VE.Direct Solar chargers 0xA040..0xA07F: 64 items */
		if (productId.value >= 0xA040 && productId.value <= 0xA07F) {
			/* Fw versions < v1.09 are not supported */
			return firmwareVersion.value >= 0x109
		}

		/* Supported: e.g. VE.Can */
		return true
	}

	model: isModelSupported() ? supportedProduct : unsupportedProduct

	/* This model is loaded when an unsupported product is connected */
	VisualItemModel {
		id: unsupportedProduct

		MbItemText {
			wrapMode: Text.WordWrap
			text: getText()

			function getText()
			{
				var text = "Unfortunately the connected MPPT Solar Charger is not compatible. ";

				switch (productId.value)
				{
				case 0x300:		/* MPPT 70/15 */
					text += "The 70/15 needs to be from year/week 1308 or later. " +
							"MPPT 70/15's currently shipped from our warehouse are compatible."
					break;
				default:
					if (firmwareVersion.value < 0x109) {
						text += "The firmware version in the MPPT Solar Charger must be v1.09 or later. " +
								"Contact Victron Service for update instructions and files."
					}
				}

				return text
			}
		}

		MbSubMenu {
			id: deviceItem
			description: qsTr("Device")
			subpage: Component {
				PageDeviceInfo {
					title: deviceItem.description
					bindPrefix: root.bindPrefix
				}
			}
		}
	}

	/* This model is loaded when an supported product is connected */
	VisualItemModel {
		id: supportedProduct

		MbItemValue {
			SystemState {
				id: state
				bind: root.service.path("/State")
			}
			description: qsTr("State")
			item.text: state.text
		}
/*
		MbItemOptions {
			description: qsTr("Switch")
			bind: root.service.path("/Mode"
			possibleValues: [
				MbOption{description: qsTr("Off"); value: 4},
				MbOption{description: qsTr("On"); value: 1}
			]
			show: valid
		}
*/
		MbItemRow {
			description: qsTr("PV")
			values: [
				/* PV voltage and current are not visible in parallel mode */
				MbTextBlock { item.bind: service.path("/Pv/V"); width: 90; visible: item.valid; height: 25 },
				MbTextBlock { item.bind: service.path("/Pv/I"); width: 90; visible: item.valid; height: 25 },
				MbTextBlock { id: yield; item.bind: service.path("/Yield/Power"); width: 90; height: 25 }
			]
		}

		MbItemRow {
			description: qsTr("Battery")
			values: [
				MbTextBlock { item.bind: service.path("/Dc/0/Voltage"); width: 90; height: 25 },
				MbTextBlock { item.bind: service.path("/Dc/0/Current"); width: 90; height: 25 },

				/* Only available on CANbus chargers */
				MbTextBlock {
					item {
						bind: service.path("/Dc/0/Temperature");
						unit: "°C"
					}
					width: 90; height: 25; visible: item.valid
				}
			]
		}

		/* This is actually the user resettable yield */
		MbItemValue {
			description: qsTr("Total yield")
			item.bind: root.service.path("/Yield/User")
		}

		MbItemValue {
			description: qsTr("System yield")
			item.bind: root.service.path("/Yield/System")
			show: item.valid
		}

		/* Only available on 15A chargers */
		MbItemOptions {
			description: qsTr("Load")
			bind: root.service.path("/Load/State")
			readonly: true
			possibleValues:[
				MbOption{description: qsTr("Off"); value: 0},
				MbOption{description: qsTr("On"); value: 1}
			]
			show: valid

			/* If load is on and current present, show current.
			 * Otherwise show the state of the load output. */
			text: valid && value && loadCurrent.valid ?
					loadCurrent.text : getText(value)

			VBusItem {
				id: loadCurrent
				bind: service.path("/Load/I")
			}
		}

		MbItemAlarm {
			description: qsTr("Low battery voltage alarm")
			bind: root.service.path("/Alarms/LowVoltage")
			show: valid
		}

		MbItemAlarm {
			description: qsTr("High battery voltage alarm")
			bind: root.service.path("/Alarms/HighVoltage")
			show: valid
		}

		MbItemChargerError {
			description: qsTr("Error")
			item.bind: root.service.path("/ErrorCode")
		}

		/* This is the master´s relay state */
		MbItemOptions {
			description: qsTr("Relay state")
			bind: root.service.path("/Relay/0/State")
			readonly: true
			possibleValues:[
				MbOption{description: qsTr("Off"); value: 0},
				MbOption{description: qsTr("On"); value: 1}
			]
			show: valid
		}

		MbSwitch {
			name: qsTr("BMS Controlled")
			bind: root.service.path("/Settings/BmsPresent")
			/// @todo EV Make this option avaiable for all installers when the BMS functionality
			/// is used.
			show: valid && ColorControl.accessLevel >= Victron.AccessService
		}

		MbItemOptions {
			description: qsTr("Parallel operation status")
			bind: root.service.path("/Link/NetworkStatus")
			readonly: true
			show: valid
			possibleValues:[
				MbOption{description: qsTr("Slave"); value: 0},
				MbOption{description: qsTr("Group Master"); value: 1},
				MbOption{description: qsTr("Instance Master"); value: 2},
				MbOption{description: qsTr("Group & Instance Master"); value: 3},
				MbOption{description: qsTr("Standalone"); value: 4}
			]
		}

		MbItemOptions {
			id: networkModeEnabled
			description: qsTr("Parallel operation")
			bind: root.service.path("/Link/NetworkMode")
			value: valid ? (item.value & 1) : localValue
			readonly: true
			show: valid
			possibleValues:[
				MbOption{description: qsTr("Off"); value: 0},
				MbOption{description: qsTr("On"); value: 1}
			]
		}

		MbItemOptions {
			id: networkModeMode
			description: qsTr("Parallel operation mode setting")
			bind: root.service.path("/Link/NetworkMode")
			value: valid ? (item.value & 0xE) : localValue
			readonly: true
			show: valid && networkModeEnabled.value
			possibleValues:[
				MbOption{description: qsTr("Standalone"); value: 0},
				MbOption{description: qsTr("Charge"); value: 2},
				MbOption{description: qsTr("Hub-1"); value: 4},
				MbOption{description: qsTr("Charge & HUB-1"); value: 6},
				MbOption{description: qsTr("BMS"); value: 8},
				MbOption{description: qsTr("Charge & BMS"); value: 0xA},
				MbOption{description: qsTr("Hub-1 & BMS"); value: 0xC},
				MbOption{description: qsTr("Charge, Hub-1 & BMS"); value: 0xE}
			]
		}

		MbItemOptions {
			description: qsTr("Parallel operation master setting")
			bind: root.service.path("/Link/NetworkMode")
			value: valid ? (item.value & 0x30) : localValue
			readonly: true
			show: valid && networkModeEnabled.value
			possibleValues:[
				MbOption{description: qsTr("Slave"); value: 0x00},
				MbOption{description: qsTr("Group master"); value: 0x10},
				MbOption{description: qsTr("Charge master"); value: 0x20},
				MbOption{description: qsTr("Group & Charge master"); value: 0x30}
			]
		}

		MbItemValue {
			description: qsTr("Parallel operation voltage")
			item.bind: root.service.path("/Link/ChargeVoltage")
			show: item.valid && networkModeEnabled.value > 0 && (networkModeMode.value & 0x04)
		}

		MbItemValue {
			description: qsTr("Parallel operation current")
			item.bind: root.service.path("/Link/ChargeCurrent")
			show: item.valid && networkModeEnabled.value > 0 && (networkModeMode.value & 0x08)
		}

		MbSubMenu {
			description: qsTr("Daily history")
			subpage: Component {
				PageSolarHistory {
					title: qsTr("Daily history")
					bindPrefix: root.bindPrefix
				}
			}
		}

		MbSubMenu {
			description: qsTr("Overall history")
			subpage: Component {
				PageSolarStats {
					title: qsTr("Overall history")
					bindPrefix: root.bindPrefix
				}
			}
		}

		MbSubMenu {
			id: supportedDeviceItem
			description: qsTr("Device")
			subpage: Component {
				PageDeviceInfo {
					title: supportedDeviceItem.description
					bindPrefix: root.bindPrefix
				}
			}
		}
	}
}
